library mvcprovider;

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class Module<M extends Model, V extends View, C extends Controller>
    extends StatelessWidget {
  final M model;
  final V view;
  final C ctrl;
  final Map<Type, dynamic> dependencies;

  Module({this.model, this.view, this.ctrl, this.dependencies});

  @override
  Widget build(BuildContext context) {
    return Material(
        child: MultiProvider(providers: [
      ChangeNotifierProvider(create: (_) => model),
      ChangeNotifierProvider(create: (_) => ctrl.init(_, model, dependencies))
    ], child: view));
  }
}

class Model extends ChangeNotifier {}

class Controller<M> extends ChangeNotifier {
  BuildContext context;
  M model;
  Map<Type, dynamic> dependencies;

  Controller init(
      BuildContext context, M model, Map<Type, dynamic> dependencies) {
    this.context = context;
    this.model = model;
    this.dependencies =
        (dependencies ?? {}).map((type, fct) => MapEntry(type, fct(context)));
    return this;
  }

  T get<T>() => dependencies[T];
}

// ignore: must_be_immutable
class View<M extends Model, C extends Controller> extends StatelessWidget {
  M model;
  C ctrl;

  @override
  Widget build(BuildContext context, [Widget child]) {
    model = Provider.of<M>(context);
    ctrl = Provider.of<Controller>(context);

    return Material(child: child);
  }
}
